@@ -116,7 +116,7 @@ mod tests {
                     },
                 )]
                 .into(),
-                submodules: Default::default(),
+                ..Default::default()
             },
             None,
         )
