@@ -2,7 +2,7 @@ use std::str::FromStr;
 use test_log::test;
 
 use cao_lang::{
-    compiler::{CallNode, IntegerNode, LaneNode, Module, StringNode, VarNode},
+    compiler::{CallNode, CardId, IntegerNode, LaneNode, Module, StringNode, VarNode},
     prelude::*,
 };
 
@@ -11,16 +11,24 @@ fn composite_card_test() {
     let cu = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
-        lanes: [(
-            "main".into(),
-            Lane::default().with_card(Card::CompositeCard {
-                name: "triplepog".to_string().into(),
-                cards: vec![
-                    Card::StringLiteral(StringNode("poggers".to_owned())),
-                    Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
-                ],
-            }),
-        )]
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
+        cards: [
+            (
+                1.into(),
+                Card::CompositeCard {
+                    name: "triplepog".to_string().into(),
+                    cards: vec![2.into(), 3.into()],
+                },
+            ),
+            (
+                2.into(),
+                Card::StringLiteral(StringNode("poggers".to_owned())),
+            ),
+            (
+                3.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+        ]
         .into(),
     };
 
@@ -41,15 +49,17 @@ fn test_trace_entry() {
         imports: Default::default(),
         submodules: Default::default(),
         lanes: [
+            ("main".into(), Lane::default().with_card(1)),
+            ("pooh".into(), Lane::default().with_card(2)),
+        ]
+        .into(),
+        cards: [
+            (1.into(), Card::Jump(LaneNode("pooh".to_owned()))),
             (
-                "main".into(),
-                Lane::default().with_card(Card::Jump(LaneNode("pooh".to_owned()))),
-            ),
-            (
-                "pooh".into(),
-                Lane::default().with_card(Card::CallNative(Box::new(CallNode(
+                2.into(),
+                Card::CallNative(Box::new(CallNode(
                     InputString::from_str("non-existent-function").unwrap(),
-                )))),
+                ))),
             ),
         ]
         .into(),
@@ -70,12 +80,17 @@ fn test_string_w_utf8() {
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
-        lanes: [(
-            "main".into(),
-            Lane::default()
-                .with_card(Card::StringLiteral(StringNode(test_str.to_string())))
-                .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("result"))),
-        )]
+        lanes: [("main".into(), Lane::default().with_card(1).with_card(2))].into(),
+        cards: [
+            (
+                1.into(),
+                Card::StringLiteral(StringNode(test_str.to_string())),
+            ),
+            (
+                2.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+        ]
         .into(),
     };
 
@@ -117,14 +132,17 @@ fn test_string_param() {
     let cu = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
-        lanes: [(
-            "main".into(),
-            Lane::default()
-                .with_card(Card::StringLiteral(StringNode(test_str.to_string())))
-                .with_card(Card::CallNative(Box::new(CallNode(
-                    InputString::from(name).unwrap(),
-                )))),
-        )]
+        lanes: [("main".into(), Lane::default().with_card(1).with_card(2))].into(),
+        cards: [
+            (
+                1.into(),
+                Card::StringLiteral(StringNode(test_str.to_string())),
+            ),
+            (
+                2.into(),
+                Card::CallNative(Box::new(CallNode(InputString::from(name).unwrap()))),
+            ),
+        ]
         .into(),
     };
 
@@ -141,21 +159,22 @@ fn simple_if_statement() {
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
-        lanes: [
+        cards: [
+            (1.into(), Card::ScalarInt(IntegerNode(42))),
+            (2.into(), Card::IfTrue(3.into())),
+            (3.into(), Card::Jump(LaneNode("pooh".to_owned()))),
+            (4.into(), Card::ScalarInt(IntegerNode(69))),
             (
-                "main".into(),
-                Lane::default()
-                    .with_card(Card::ScalarInt(IntegerNode(42)))
-                    .with_card(Card::IfTrue(Box::new(Card::Jump(LaneNode(
-                        "pooh".to_owned(),
-                    ))))),
+                5.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
             ),
+        ]
+        .into(),
+        lanes: [
+            ("main".into(), Lane::default().with_card(1).with_card(2)),
             (
                 "pooh".into(),
-                Lane::default().with_cards(vec![
-                    Card::ScalarInt(IntegerNode(69)),
-                    Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
-                ]),
+                Lane::default().with_cards(vec![4.into(), CardId(5)]),
             ),
         ]
         .into(),
@@ -176,25 +195,44 @@ fn simple_if_statement_skips_if_false() {
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::ScalarInt(IntegerNode(0))),
+            (
+                2.into(),
+                Card::CompositeCard {
+                    name: None,
+                    cards: vec![3.into(), 4.into()],
+                },
+            ),
+            (3.into(), Card::ScalarInt(IntegerNode(69))),
+            (
+                4.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+            (5.into(), Card::IfTrue(2.into())),
+            (6.into(), Card::ScalarInt(IntegerNode(1))),
+            (7.into(), Card::IfFalse(8.into())),
+            (
+                8.into(),
+                Card::CompositeCard {
+                    name: None,
+                    cards: vec![9.into(), 10.into()],
+                },
+            ),
+            (9.into(), Card::ScalarInt(IntegerNode(42))),
+            (
+                10.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+        ]
+        .into(),
         lanes: [(
             "main".into(),
             Lane::default()
-                .with_card(Card::ScalarInt(IntegerNode(0)))
-                .with_card(Card::IfTrue(Box::new(Card::CompositeCard {
-                    name: None,
-                    cards: vec![
-                        Card::ScalarInt(IntegerNode(69)),
-                        Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
-                    ],
-                })))
-                .with_card(Card::ScalarInt(IntegerNode(1)))
-                .with_card(Card::IfFalse(Box::new(Card::CompositeCard {
-                    name: None,
-                    cards: vec![
-                        Card::ScalarInt(IntegerNode(42)),
-                        Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
-                    ],
-                }))),
+                .with_card(1)
+                .with_card(5)
+                .with_card(6)
+                .with_card(7),
         )]
         .into(),
     };
@@ -218,32 +256,41 @@ fn if_else_test(condition: Card, true_res: Card, false_res: Card, expected_resul
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), condition),
+            (
+                2.into(),
+                Card::IfElse {
+                    then: 5.into(),
+                    r#else: 6.into(),
+                },
+            ),
+            (3.into(), Card::ScalarInt(IntegerNode(0xbeef))),
+            (
+                4.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result2")),
+            ),
+            (5.into(), Card::Jump(LaneNode("pooh".to_string()))),
+            (6.into(), Card::Jump(LaneNode("tiggers".to_string()))),
+            (7.into(), true_res),
+            (
+                8.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+            (9.into(), false_res),
+        ]
+        .into(),
         lanes: [
             (
                 "main".into(),
                 Lane::default()
-                    .with_card(condition)
-                    .with_card(Card::IfElse {
-                        then: Box::new(Card::Jump(LaneNode("pooh".to_string()))),
-                        r#else: Box::new(Card::Jump(LaneNode("tiggers".to_string()))),
-                    })
-                    .with_card(Card::ScalarInt(IntegerNode(0xbeef)))
-                    .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("result2"))),
-            ),
-            (
-                "pooh".into(),
-                Lane::default().with_cards(vec![
-                    true_res,
-                    Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
-                ]),
-            ),
-            (
-                "tiggers".into(),
-                Lane::default().with_cards(vec![
-                    false_res,
-                    Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
-                ]),
+                    .with_card(1)
+                    .with_card(2)
+                    .with_card(3)
+                    .with_card(4),
             ),
+            ("pooh".into(), Lane::default().with_card(7).with_card(8)),
+            ("tiggers".into(), Lane::default().with_card(9).with_card(8)),
         ]
         .into(),
     };
@@ -290,18 +337,33 @@ fn test_local_variable() {
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::ScalarInt(IntegerNode(420))),
+            (
+                2.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("bar")),
+            ),
+            (3.into(), Card::ScalarInt(IntegerNode(123))),
+            (4.into(), Card::SetVar(VarNode::from_str_unchecked("foo"))),
+            (5.into(), Card::ReadVar(VarNode::from_str_unchecked("foo"))),
+            (
+                6.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("bar")),
+            ),
+        ]
+        .into(),
         lanes: [(
             "main".into(),
             Lane::default()
                 // init the global variable
-                .with_card(Card::ScalarInt(IntegerNode(420)))
-                .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("bar")))
+                .with_card(1)
+                .with_card(2)
                 // set another value in local var
-                .with_card(Card::ScalarInt(IntegerNode(123)))
-                .with_card(Card::SetVar(VarNode::from_str_unchecked("foo")))
+                .with_card(3)
+                .with_card(4)
                 // read the var and set the global variable
-                .with_card(Card::ReadVar(VarNode::from_str_unchecked("foo")))
-                .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("bar"))),
+                .with_card(5)
+                .with_card(6),
         )]
         .into(),
     };
@@ -324,18 +386,19 @@ fn local_variable_doesnt_leak_out_of_scope() {
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::ScalarInt(IntegerNode(123))),
+            (2.into(), Card::SetVar(VarNode::from_str_unchecked("foo"))),
+            (3.into(), Card::Jump(LaneNode("bar".to_string()))),
+            (4.into(), Card::ReadVar(VarNode::from_str_unchecked("foo"))),
+        ]
+        .into(),
         lanes: [
             (
                 "main".into(),
-                Lane::default()
-                    .with_card(Card::ScalarInt(IntegerNode(123)))
-                    .with_card(Card::SetVar(VarNode::from_str_unchecked("foo")))
-                    .with_card(Card::Jump(LaneNode("bar".to_string()))),
-            ),
-            (
-                "bar".into(),
-                Lane::default().with_card(Card::ReadVar(VarNode::from_str_unchecked("foo"))),
+                Lane::default().with_card(1).with_card(2).with_card(3),
             ),
+            ("bar".into(), Lane::default().with_card(4)),
         ]
         .into(),
     };
@@ -358,23 +421,40 @@ fn simple_while_loop() {
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::ScalarInt(IntegerNode(69))),
+            (
+                2.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+            (3.into(), Card::While(LaneNode("pooh".to_string()))),
+            (
+                4.into(),
+                Card::ReadVar(VarNode::from_str_unchecked("result")),
+            ),
+            (5.into(), Card::ScalarInt(IntegerNode(1))),
+            (6.into(), Card::Sub),
+            (7.into(), Card::CopyLast), // return `result`
+            (
+                8.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+        ]
+        .into(),
         lanes: [
             (
                 "main".into(),
-                Lane::default()
-                    .with_card(Card::ScalarInt(IntegerNode(69)))
-                    .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("result")))
-                    .with_card(Card::While(LaneNode("pooh".to_string()))),
+                Lane::default().with_card(1).with_card(2).with_card(3),
             ),
             (
                 "pooh".into(),
                 Lane::default().with_cards(vec![
                     // Add 1 to the global 'result' variable in each iteration
-                    Card::ReadVar(VarNode::from_str_unchecked("result")),
-                    Card::ScalarInt(IntegerNode(1)),
-                    Card::Sub,
-                    Card::CopyLast, // return `result`
-                    Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+                    4.into(),
+                    5.into(),
+                    6.into(),
+                    7.into(),
+                    8.into(),
                 ]),
             ),
         ]
@@ -407,26 +487,39 @@ fn simple_for_loop() {
     let program = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::ScalarInt(IntegerNode(0))),
+            (
+                2.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+            (3.into(), Card::ScalarInt(IntegerNode(5))),
+            (4.into(), Card::Repeat(LaneNode("Loop".to_string()))),
+            (5.into(), Card::ReadVar(VarNode::from_str_unchecked("i"))),
+            (
+                6.into(),
+                Card::ReadVar(VarNode::from_str_unchecked("result")),
+            ),
+            (7.into(), Card::Add),
+            (
+                8.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+            ),
+        ]
+        .into(),
         lanes: [
             (
                 "main".into(),
-                Lane::default().with_cards(vec![
-                    // init the result variable
-                    Card::ScalarInt(IntegerNode(0)),
-                    Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
-                    // loop
-                    Card::ScalarInt(IntegerNode(5)),
-                    Card::Repeat(LaneNode("Loop".to_string())),
-                ]),
+                Lane::default().with_cards(vec![1.into(), 2.into(), 3.into(), 4.into()]),
             ),
             (
                 "Loop".into(),
                 Lane::default().with_arg("i").with_cards(vec![
                     // Add i to the global 'result' variable in each iteration
-                    Card::ReadVar(VarNode::from_str_unchecked("i")),
-                    Card::ReadVar(VarNode::from_str_unchecked("result")),
-                    Card::Add,
-                    Card::SetGlobalVar(VarNode::from_str_unchecked("result")),
+                    5.into(),
+                    6.into(),
+                    7.into(),
+                    8.into(),
                 ]),
             ),
         ]
@@ -451,13 +544,12 @@ fn call_native_test() {
     let cu = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
-        lanes: [(
-            "main".into(),
-            Lane::default().with_cards(vec![Card::CallNative(Box::new(CallNode(
-                InputString::from(name).unwrap(),
-            )))]),
+        cards: [(
+            1.into(),
+            Card::CallNative(Box::new(CallNode(InputString::from(name).unwrap()))),
         )]
         .into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
     };
 
     let prog = compile(cu, CompileOptions::new()).unwrap();
@@ -529,30 +621,22 @@ fn test_function_registry() {
     const PROG: &str = r#"
 submodules: {}
 imports: []
+cards:
+    1: !CallNative "func0"
+    2: !ScalarInt 42
+    3: !CallNative "func1"
+    4: !ScalarInt 12
+    5: !ScalarFloat 4.2
+    6: !CallNative "func2"
+    7: !ScalarInt 33
+    8: !ScalarFloat 2.88
+    9: !ScalarInt 0
+    10: !CallNative "func3"
 lanes:
     main:
         name: main
-        cards:
-            - ty: CallNative
-              val: "func0"
-            - ty: ScalarInt
-              val: 42
-            - ty: CallNative
-              val: "func1"
-            - ty: ScalarInt
-              val: 12
-            - ty: ScalarFloat
-              val: 4.2
-            - ty: CallNative
-              val: "func2"
-            - ty: ScalarInt
-              val: 33
-            - ty: ScalarFloat
-              val: 2.88
-            - ty: ScalarInt
-              val: 0
-            - ty: CallNative
-              val: "func3"
+        arguments: []
+        cards: [1,2,3,4,5,6,7,8,9,10]
 "#;
     let cu = serde_yaml::from_str(PROG).unwrap();
     let prog = compile(cu, CompileOptions::new()).unwrap();
@@ -571,25 +655,39 @@ fn jump_lane_w_params_test() {
     let cu = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::ScalarInt(IntegerNode(42))),
+            (
+                2.into(),
+                Card::StringLiteral(StringNode("winnie the pooh".to_owned())),
+            ),
+            (3.into(), Card::Jump(LaneNode("pooh".to_owned()))),
+            (4.into(), Card::ReadVar(VarNode::from_str_unchecked("foo"))),
+            (
+                5.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("g_foo")),
+            ),
+            (6.into(), Card::ReadVar(VarNode::from_str_unchecked("bar"))),
+            (
+                7.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("g_bar")),
+            ),
+        ]
+        .into(),
         lanes: [
             (
                 "main".into(),
-                Lane::default()
-                    .with_card(Card::ScalarInt(IntegerNode(42)))
-                    .with_card(Card::StringLiteral(StringNode(
-                        "winnie the pooh".to_owned(),
-                    )))
-                    .with_card(Card::Jump(LaneNode("pooh".to_owned()))),
+                Lane::default().with_card(1).with_card(2).with_card(3),
             ),
             (
                 "pooh".into(),
                 Lane::default()
                     .with_arg("foo")
                     .with_arg("bar")
-                    .with_card(Card::ReadVar(VarNode::from_str_unchecked("foo")))
-                    .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_foo")))
-                    .with_card(Card::ReadVar(VarNode::from_str_unchecked("bar")))
-                    .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_bar"))),
+                    .with_card(4)
+                    .with_card(5)
+                    .with_card(6)
+                    .with_card(7),
             ),
         ]
         .into(),
@@ -622,12 +720,18 @@ fn len_test_empty() {
     let cu = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::CreateTable),
+            (2.into(), Card::Len),
+            (
+                3.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")),
+            ),
+        ]
+        .into(),
         lanes: [(
             "main".into(),
-            Lane::default()
-                .with_card(Card::CreateTable)
-                .with_card(Card::Len)
-                .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_result"))),
+            Lane::default().with_card(1).with_card(2).with_card(3),
         )]
         .into(),
     };
@@ -651,30 +755,35 @@ fn len_test_happy() {
     let cu = CaoProgram {
         imports: Default::default(),
         submodules: Default::default(),
+        cards: [
+            (1.into(), Card::CreateTable),
+            (2.into(), Card::SetVar(t.clone())),
+            (3.into(), Card::ReadVar(t.clone())),
+            (4.into(), Card::StringLiteral(StringNode("asd".to_string()))),
+            (5.into(), Card::ScalarInt(IntegerNode(42))),
+            (6.into(), Card::SetProperty),
+            (7.into(), Card::ReadVar(t.clone())),
+            (8.into(), Card::StringLiteral(StringNode("asd".to_string()))),
+            (9.into(), Card::ScalarInt(IntegerNode(42))),
+            (10.into(), Card::SetProperty),
+            (11.into(), Card::ReadVar(t.clone())),
+            (
+                12.into(),
+                Card::StringLiteral(StringNode("basdasd".to_string())),
+            ),
+            (13.into(), Card::ScalarInt(IntegerNode(42))),
+            (14.into(), Card::SetProperty),
+            (15.into(), Card::ReadVar(t.clone())),
+            (16.into(), Card::Len),
+            (
+                17.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")),
+            ),
+        ]
+        .into(),
         lanes: [(
             "main".into(),
-            Lane::default()
-                .with_card(Card::CreateTable)
-                .with_card(Card::SetVar(t.clone()))
-                // first property
-                .with_card(Card::ReadVar(t.clone()))
-                .with_card(Card::StringLiteral(StringNode("asd".to_string())))
-                .with_card(Card::ScalarInt(IntegerNode(42)))
-                .with_card(Card::SetProperty)
-                // same property as above
-                .with_card(Card::ReadVar(t.clone()))
-                .with_card(Card::StringLiteral(StringNode("asd".to_string())))
-                .with_card(Card::ScalarInt(IntegerNode(42)))
-                .with_card(Card::SetProperty)
-                // new property
-                .with_card(Card::ReadVar(t.clone()))
-                .with_card(Card::StringLiteral(StringNode("basdasd".to_string())))
-                .with_card(Card::ScalarInt(IntegerNode(42)))
-                .with_card(Card::SetProperty)
-                // len
-                .with_card(Card::ReadVar(t.clone()))
-                .with_card(Card::Len)
-                .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_result"))),
+            Lane::default().with_cards((1..=17).map(Into::into).collect::<Vec<_>>()),
         )]
         .into(),
     };
@@ -700,27 +809,28 @@ fn nested_module_can_call_self_test() {
             Module {
                 imports: Default::default(),
                 submodules: Default::default(),
+                cards: [
+                    (1.into(), Card::Jump(LaneNode("nie".to_owned()))),
+                    (
+                        2.into(),
+                        Card::StringLiteral(StringNode("poggers".to_owned())),
+                    ),
+                    (
+                        3.into(),
+                        Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")),
+                    ),
+                ]
+                .into(),
                 lanes: [
-                    (
-                        "win".into(),
-                        Lane::default().with_card(Card::Jump(LaneNode("nie".to_owned()))),
-                    ),
-                    (
-                        "nie".into(),
-                        Lane::default()
-                            .with_card(Card::StringLiteral(StringNode("poggers".to_owned())))
-                            .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_result"))),
-                    ),
+                    ("win".into(), Lane::default().with_card(1)),
+                    ("nie".into(), Lane::default().with_card(2).with_card(3)),
                 ]
                 .into(),
             },
         )]
         .into(),
-        lanes: [(
-            "main".into(),
-            Lane::default().with_card(Card::Jump(LaneNode("winnie.win".to_owned()))),
-        )]
-        .into(),
+        cards: [(1.into(), Card::Jump(LaneNode("winnie.win".to_owned())))].into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
     };
 
     let program = compile(cu, CompileOptions::new()).expect("compile");
@@ -742,11 +852,8 @@ fn invalid_import_is_error_test() {
     let cu = CaoProgram {
         submodules: [].into(),
         imports: ["winnie..pooh".into()].into(),
-        lanes: [(
-            "main".into(),
-            Lane::default().with_card(Card::Jump(LaneNode("pooh".to_owned()))),
-        )]
-        .into(),
+        cards: [(1.into(), Card::Jump(LaneNode("pooh".to_owned())))].into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
     };
 
     compile(cu, CompileOptions::new()).expect_err("compile");
@@ -757,11 +864,8 @@ fn non_existent_import_is_error_test() {
     let cu = CaoProgram {
         submodules: [].into(),
         imports: ["winnie..pooh".into()].into(),
-        lanes: [(
-            "main".into(),
-            Lane::default().with_card(Card::Jump(LaneNode("pooh".to_owned()))),
-        )]
-        .into(),
+        cards: [(1.into(), Card::Jump(LaneNode("pooh".to_owned())))].into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
     };
 
     let err = compile(cu, CompileOptions::new()).expect_err("compile");
@@ -782,31 +886,190 @@ fn import_in_submodule_test() {
             Module {
                 imports: Default::default(),
                 submodules: Default::default(),
-                lanes: [(
-                    "pooh".into(),
-                    Lane::default()
-                        .with_card(Card::StringLiteral(StringNode("poggers".to_owned())))
-                        .with_card(Card::SetGlobalVar(VarNode::from_str_unchecked("g_result"))),
-                )]
+                cards: [
+                    (
+                        1.into(),
+                        Card::StringLiteral(StringNode("poggers".to_owned())),
+                    ),
+                    (
+                        2.into(),
+                        Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")),
+                    ),
+                ]
                 .into(),
+                lanes: [("pooh".into(), Lane::default().with_card(1).with_card(2))].into(),
             },
         )]
         .into(),
         imports: ["winnie.pooh".into()].into(),
-        lanes: [(
-            "run".into(),
-            Lane::default().with_card(Card::Jump(LaneNode("pooh".to_owned()))),
-        )]
-        .into(),
+        cards: [(1.into(), Card::Jump(LaneNode("pooh".to_owned())))].into(),
+        lanes: [("run".into(), Lane::default().with_card(1))].into(),
     };
     let cu = Module {
         imports: ["foo.run".into()].into(),
         submodules: [("foo".into(), cu)].into(),
-        lanes: [(
-            "main".into(),
-            Lane::default().with_card(Card::Jump(LaneNode("run".to_owned()))),
+        cards: [(1.into(), Card::Jump(LaneNode("run".to_owned())))].into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("g_result", &program.variables)
+        .expect("Failed to read foo variable");
+
+    let result = unsafe { result.as_str().unwrap() };
+
+    assert_eq!(result, "poggers");
+}
+
+#[test]
+fn can_import_submodule_test() {
+    let cu = Module {
+        submodules: [(
+            "winnie".into(),
+            Module {
+                imports: Default::default(),
+                submodules: Default::default(),
+                cards: [
+                    (
+                        1.into(),
+                        Card::StringLiteral(StringNode("poggers".to_owned())),
+                    ),
+                    (
+                        2.into(),
+                        Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")),
+                    ),
+                ]
+                .into(),
+                lanes: [("pooh".into(), Lane::default().with_card(1).with_card(2))].into(),
+            },
         )]
         .into(),
+        imports: ["winnie.pooh".into()].into(),
+        cards: [(1.into(), Card::Jump(LaneNode("pooh".to_owned())))].into(),
+        lanes: [("run".into(), Lane::default().with_card(1))].into(),
+    };
+    let cu = Module {
+        imports: ["foo.winnie".into()].into(),
+        submodules: [("foo".into(), cu)].into(),
+        cards: [(1.into(), Card::Jump(LaneNode("winnie.pooh".to_owned())))].into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("g_result", &program.variables)
+        .expect("Failed to read foo variable");
+
+    let result = unsafe { result.as_str().unwrap() };
+
+    assert_eq!(result, "poggers");
+}
+
+#[test]
+fn can_import_function_from_super_module_test() {
+    let winnie = Module {
+        submodules: Default::default(),
+        cards: [(1.into(), Card::Jump(LaneNode("pog".to_owned())))].into(),
+        lanes: [("pooh".into(), Lane::default().with_card(1))].into(),
+        imports: ["super.super.pog".into()].into(),
+    };
+    let foo = Module {
+        submodules: [("winnie".into(), winnie)].into(),
+        imports: Default::default(),
+        cards: [].into(),
+        lanes: [].into(),
+    };
+    let bar = Module {
+        imports: ["foo.winnie".into()].into(),
+        submodules: [("foo".into(), foo)].into(),
+        cards: [
+            (1.into(), Card::Jump(LaneNode("winnie.pooh".to_owned()))),
+            (
+                2.into(),
+                Card::StringLiteral(StringNode("poggers".to_owned())),
+            ),
+            (
+                3.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")),
+            ),
+        ]
+        .into(),
+        lanes: [
+            ("run".into(), Lane::default().with_card(1)),
+            ("pog".into(), Lane::default().with_card(2).with_card(3)),
+        ]
+        .into(),
+    };
+    let cu = Module {
+        imports: [].into(),
+        submodules: [("bar".into(), bar)].into(),
+        cards: [(1.into(), Card::Jump(LaneNode("bar.run".to_owned())))].into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
+    };
+
+    let program = compile(cu, CompileOptions::new()).expect("compile");
+
+    let mut vm = Vm::new(()).unwrap();
+    vm.run(&program).expect("run");
+
+    let result = vm
+        .read_var_by_name("g_result", &program.variables)
+        .expect("Failed to read foo variable");
+
+    let result = unsafe { result.as_str().unwrap() };
+
+    assert_eq!(result, "poggers");
+}
+
+#[test]
+fn import_super_module_test() {
+    let winnie = Module {
+        imports: ["super.super.bar".into()].into(),
+        submodules: Default::default(),
+        cards: [(1.into(), Card::Jump(LaneNode("bar.pog".to_owned())))].into(),
+        lanes: [("pooh".into(), Lane::default().with_card(1))].into(),
+    };
+    let foo = Module {
+        submodules: [("winnie".into(), winnie)].into(),
+        imports: Default::default(),
+        cards: [].into(),
+        lanes: [].into(),
+    };
+    let bar = Module {
+        imports: ["foo.winnie".into()].into(),
+        submodules: [("foo".into(), foo)].into(),
+        cards: [
+            (1.into(), Card::Jump(LaneNode("winnie.pooh".to_owned()))),
+            (
+                2.into(),
+                Card::StringLiteral(StringNode("poggers".to_owned())),
+            ),
+            (
+                3.into(),
+                Card::SetGlobalVar(VarNode::from_str_unchecked("g_result")),
+            ),
+        ]
+        .into(),
+        lanes: [
+            ("run".into(), Lane::default().with_card(1)),
+            ("pog".into(), Lane::default().with_card(2).with_card(3)),
+        ]
+        .into(),
+    };
+    let cu = Module {
+        imports: [].into(),
+        submodules: [("bar".into(), bar)].into(),
+        cards: [(1.into(), Card::Jump(LaneNode("bar.run".to_owned())))].into(),
+        lanes: [("main".into(), Lane::default().with_card(1))].into(),
     };
 
     let program = compile(cu, CompileOptions::new()).expect("compile");
