@@ -10,26 +10,30 @@ def test_compile_and_run():
     """
 
     program_yaml = """
+cards:
+            1: !ScalarInt 5
+            2: !Add
+            3: !Jump "foo.bar"
 lanes:
     main: 
+        arguments: []
         cards:
-            - ty: ScalarInt
-              val: 5
-            - ty: ScalarInt
-              val: 5
-            - ty: Add
-            - ty: Jump
-              val: "foo.bar"
+            - 1
+            - 1
+            - 2
+            - 3
 imports: []
 submodules:
     foo:
         imports: []
         submodules: {}
+        cards:
+            1: !ScalarInt 42
         lanes:
             bar:
+                arguments: []
                 cards:
-                    - ty: ScalarInt
-                      val: 42
+                    - 1
 
 """
 
@@ -49,11 +53,13 @@ def test_get_version():
 def test_json():
     program_json = """
     {
+        "cards": {
+            "1": {  "Jump": "foo.bar" }
+        },
         "lanes": {
             "main": {
-                "cards": [
-                    { "ty": "Jump", "val": "foo.bar" }
-                ]
+                "arguments": [],
+                "cards": [ 1 ]
             }
         },
         "imports": [],
@@ -61,11 +67,13 @@ def test_json():
             "foo": {
                 "imports": [],
                 "submodules": {},
+                "cards": {
+                    "1": {  "Noop":null }
+                },
                 "lanes": {
                     "bar": {
-                        "cards": [
-                            { "ty": "Noop" }
-                        ]
+                        "arguments": [],
+                        "cards": [ 1 ]
                     }
                 }
             }
@@ -95,10 +103,20 @@ def test_bad_json_is_value_error():
 
 
 def test_recursion_limit():
-    program = {"imports": [], "submodules": {}, "lanes": {"main": {"cards": []}}}
+    program = {
+        "imports": [],
+        "submodules": {},
+        "cards": {},
+        "lanes": {"main": {"arguments": [], "cards": []}},
+    }
     _pr = program
     for _ in range(2):
-        _pr["submodules"]["foo"] = {"imports": [], "submodules": {}, "lanes": {}}
+        _pr["submodules"]["foo"] = {
+            "imports": [],
+            "submodules": {},
+            "lanes": {},
+            "cards": {},
+        }
         _pr = _pr["submodules"]["foo"]
 
     program = caoc.CompilationUnit.from_json(json.dumps(program))
